/* -*- C++ -*-
   (c) 2008 Peter Rockai <me@mornfall.net>
   (c) 2008 Yuriy Kozlov <yuriy.kozlov@gmail.com> */

#include <adept/packagedetails.h>

#include <KDirLister>
#include <fstream>

using namespace adept;

PackageDetails::PackageDetails( package::Source &_p, record::Source &_r,
                                package::ActionList &_actl,
                    QWidget *parent )
    : PackageInfo(_p, _r, _actl, parent)
{
    m_layout = new QVBoxLayout( this );

    setupInfo();
    setupTabWidget();

    m_needExpensiveRefresh = false;
    
    connect( this, SIGNAL( changed() ), this, SLOT( refresh() ) );
}

void PackageDetails::loadFileList() {
    std::string fl, flfile = "/var/lib/dpkg/info/" + t.package() + ".list";
    std::ifstream ifl( flfile.c_str() );

    int i = 0;
    qDebug() << "PackageDetails::loadFileListWorker() entering loop" << endl;
    m_fileProxy->clear();

    bool first = true;
    while ( ifl.is_open() && !ifl.eof() ) {
        std::string line;
        getline( ifl, line );
        if ( line == "/." || line == "" )
            continue; // nasty evil thing go away
        KUrl url( "file:///" );
        url.addPath( u8( line ) );
        qDebug() << url;
        m_fileProxy->addUrl( url );
        first = false;
        ++i;
    }
    m_fileProxy->apply();
    qDebug() << "PackageDetails::loadFileListWorker() leaving loop" << endl;
    ifl.close();
    m_fileList->expandToDepth( 2 );
}
    
void PackageDetails::refresh(){
    block = true;
    setUpdatesEnabled( false );
    m_needExpensiveRefresh = true;

    PackageState s = p.get< package::State >( t );
    
    QString name = u8( t.package() ) + ": ",
            status = statusString( s ),
            comma = u8( " — " ),
            action = actionString( s ),
            shortdesc = u8( r.get< record::ShortDescription >( t ) );
    QString nameStatusLabelStr = "<b>" + name + "</b>" + 
        "<font color=" + statusColor( s ).name() + ">" + status + "</font>";
    if ( !s.keep() ) {
        nameStatusLabelStr += comma
                + "<font color=" + actionColor( s ).name() + ">" + action + "</font>";
    }
    m_pkgNameLabel->setText(nameStatusLabelStr);
    m_shortDesc->setText(shortdesc);
            
    QString versionInfo;
    ept::Token candidate = p.get< package::CandidateVersion >( t ),
                installed = p.get< package::InstalledVersion >( t );
    if ( s.upgradable() ) {
        versionInfo = i18n( "<nobr>Version<b>" )
                + u8( installed.version() )
                + i18n( "</b> is installed,</nobr> <nobr>version <b>" )
                + u8( candidate.version() )
                + i18n( "</b> is available.</nobr>" );
    } else if ( !s.installed() && p.exists( candidate ) ) {
        versionInfo = i18n( "<nobr>Version <b>" )
                + u8( candidate.version() )
                + i18n( "</b> is available.</nobr>" );
    } else if ( p.exists( installed ) ) {
        versionInfo = i18n( "<nobr>Version <b>" )
                + u8( installed.version() )
                + i18n( "</b> is installed.</nobr>" );
    } else {
        versionInfo = i18n( "<nobr>No versions available.</nobr>" );
    }

    QString maintainer = i18n( "Maintainer: <b>" )
                         + u8( r.get< record::Maintainer >( t ) )
                         + i18n( "</b>" ),
               section = i18n( "Section: <b>" )
                         + u8( r.get< record::Section >( t ) )
                         + i18n( "</b>" ),
         installedSize = i18n( "Installed Size: <b>" )
                         + QString::number(
                             r.get< record::InstalledSize >( t ) ) + "kB"
                         + i18n( "</b>" ),
         sourcePackage = i18n( "Source Package: <i>" )
                         + u8( r.get< record::SourcePackage >( t ) )
                         + i18n( "</i>" ),
          architecture = i18n( "Architecture: <b>" )
                         + u8( r.get< record::Architecture >( t ) )
                         + i18n( "</b>" );

    m_desc->setText( formatLongDescription(
                            u8( r.get< record::LongDescription >( t ) ) ) );
    m_versionInfo->setText( versionInfo );
    m_maintainer->setText( maintainer );
    m_section->setText( section );
    m_installedSize->setText( installedSize );
    m_sourcePackage->setText( sourcePackage );
    m_architecture->setText( architecture );

    if ( !m_act ) {
        m_act = new QCheckBox( m_infoBox );
        connect( m_act, SIGNAL( stateChanged( int ) ), this, 
                    SLOT( actStateChanged( int ) ) );
    }

    if ( s.upgradable() ) {
        m_act->setText( i18n( "Upgrade this package" ) );
        if ( !m_remove ) {
            m_remove = new QCheckBox( m_infoBox );
            m_remove->setText( i18n( "Remove this package" ) );
            connect( m_remove, SIGNAL( stateChanged( int ) ), this, 
                        SLOT( removeStateChanged( int ) ) );
        }
        m_act->setChecked( s.upgrade() );
        m_remove->setChecked( s.remove() );
    } else if ( s.installed() ) {
        delete m_remove;
        m_remove = 0;
        m_act->setText( i18n( "Remove this package" ) );
        m_act->setChecked( s.remove() );
    } else {
        delete m_remove;
        m_remove = 0;
        m_act->setText( i18n( "Install this package" ) );
        m_act->setChecked( s.install() );
    }
    
    setUpdatesEnabled( true );
    block = false;
    update();
}
    
#include "packagedetails.moc"
