/* -*- C++ -*-
   (c) 2008 Peter Rockai <me@mornfall.net>,
   (c) 2008 Yuriy Kozlov <yuriy.kozlov@gmail.com> */

#include <QtGui/QWidget>
#include <QtGui/QCheckBox>

#include <QtCore/QRegExp>
#include <QtCore/QDebug>

#include <ept/core/apt.h>
#include <ept/core/apt/action.h>

#include <adept/util.h>

#ifndef ADEPTPACKAGEINFO_H
#define ADEPTPACKAGEINFO_H

namespace adept {

using namespace ept::core;

/**
 * @class PackageInfo
 * Abstract class with common fields and methods for displaying
 * package information.  Provides a checkboxes and methods for
 * changing the package state.
 */
class PackageInfo : public QWidget
{
    Q_OBJECT
protected:
    package::Source &p;
    record::Source &r;
    package::ActionList &actl;
    ept::Token t;
    
    bool block;
    
    QCheckBox *m_act, *m_remove;
    
public:
    PackageInfo(package::Source &_p, record::Source &_r,
                package::ActionList &_actl,
                    QWidget *parent = 0 )
        : QWidget( parent ), m_act( 0 ), m_remove( 0 ),
                p( _p ), r( _r ), actl( _actl ), block( false )
    {
    }
    
    PackageInfo( package::Source &_p, record::Source &_r,
                     package::ActionList &_actl, ept::Token _t,
                     QWidget *parent = 0 )
        : QWidget( parent ), m_act( 0 ), m_remove( 0 ),
               p( _p ), r( _r ), actl(_actl), t(_t), block( false )
    {
    }

    ~PackageInfo() {};
    
public Q_SLOTS:
    virtual void refresh() = 0;

    void removeStateChanged( int s ) {
        if ( block )
            return;
        block = true;
        if ( s == Qt::Unchecked ) {
            actl.add( package::Action( t, package::Action::Keep ) );
        } else {
            m_act->setChecked( false );
            actl.add( package::Action( t, package::Action::Remove ) );
        }
        p.revertStates();
        actl.replay( p );
        block = false;
        changed();
    }

    void actStateChanged( int s ) {
        if ( block )
            return;
        block = true;
        if ( s == Qt::Unchecked ) {
            actl.add( package::Action( t, package::Action::Keep ) );
        } else {
            PackageState st = p.get< package::State >( t );
            if (m_remove)
                m_remove->setChecked( false );
            if ( st.installed() && !st.upgradable() )
                actl.add( package::Action( t, package::Action::Remove ) );
            else
                actl.add( package::Action( t, package::Action::Install ) );
        }
        p.revertStates();
        actl.replay( p );
        block = false;
        changed();
    }
    
Q_SIGNALS:
    void changed();

};

}

#endif
